/***************************************************************************
 *
 *  $Id: kzenconfigdialog.cpp,v 1.10 2005/06/01 08:04:20 muszilla Exp $
 *
 *  Copyright (C) 2005 by Andreas Mussgiller
 *  muszilla@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the
 *  Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ***************************************************************************/

#include <qiconset.h>
#include <qpixmap.h>
#include <qtooltip.h>
#include <qvgroupbox.h>
#include <qcheckbox.h>

#include <kjanuswidget.h>
#include <klocale.h>
#include <kiconloader.h>
#include <klistbox.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <kdatetimewidget.h>
#include <qlabel.h>
#include <kdebug.h>
#include <kdirselectdialog.h>

#include <qvbox.h>
#include <qhbox.h>

#include "kzenexplorer.h"
#include "kzencategories.h"
#include "kzenexplorerview.h"
#include "kzentrack.h"
#include "kzenconfig.h"

#include "kzenconfigdialog.h"

KZenConfigDialog::KZenConfigDialog(QWidget* parent, const char* name)
  :KDialogBase(KDialogBase::IconList,WStyle_DialogBorder, parent, name,
	       true, i18n("Configuration - KZenExplorer"), Ok|Cancel)
{
  fBusy = true;

  fExplorer = (KZenExplorer*) parent;
 
  QVBox * jukeboxPage = addVBoxPage(i18n("Jukebox"),i18n("Jukebox"),
				    UserIcon("jukebox", 16));

  QHBox * ownerBox = new QHBox(jukeboxPage);

  ownerBox->setSpacing(5);
  QLabel * label = new QLabel(i18n("Owner:"),ownerBox);
  fOwnerEdit = new KLineEdit(ownerBox,"lineedit");
  connect(fOwnerEdit,SIGNAL(textChanged(const QString &)),
	  this,SLOT(ownerChanged(const QString &)));
  jukeboxPage->setStretchFactor(ownerBox, 0);

  QHBox * clockBox = new QHBox(jukeboxPage);
  
  clockBox->setSpacing(5);
  label = new QLabel(i18n("Clock:"),clockBox);
  fClockWidget = new KDateTimeWidget(clockBox,"ClockWidget");
  connect(fClockWidget,SIGNAL(valueChanged(const QDateTime &)),
	  this,SLOT(clockChanged(const QDateTime &)));
  fAdjustClock = new KPushButton(i18n("Adjust"),clockBox,"AdjustClock");
  connect(fAdjustClock,SIGNAL(clicked()),
	  this,SLOT(clockAdjust()));
  jukeboxPage->setStretchFactor(clockBox, 0);

  fSmartImport = new QCheckBox(i18n("Smart Import of Tracks and Playlists"),
			       jukeboxPage,"SmartImport");

  jukeboxPage->setStretchFactor(new QWidget(jukeboxPage),1);

  QHBox * categoryPage = addHBoxPage(i18n("Categories"),i18n("Categories"),
				     UserIcon("category", 16));

  QVBox * categoryLists = new QVBox(categoryPage);

  fCategoryEdit = new KLineEdit(categoryLists,"CategoryEdit");

  fCategoryListBox = new KListBox(categoryLists,"CategoryListBox");
  fCategoryListBox->setMinimumHeight(150);
  fCategoryListBox->insertStringList(gCategories->categories());

  QVBox * categoryButtons = new QVBox(categoryPage);

  fAddCategory = new KPushButton(i18n("Add"),categoryButtons,"AddCategory");
  categoryButtons->setStretchFactor(fAddCategory, 0);
  fModifyCategory = new KPushButton(i18n("Modify"),categoryButtons,"ModifyCategory");
  categoryButtons->setStretchFactor(fModifyCategory, 0);
  fRemoveCategory = new KPushButton(i18n("Remove"),categoryButtons,"RemoveCategory");
  categoryButtons->setStretchFactor(fRemoveCategory, 0);
  categoryButtons->setStretchFactor(new QWidget(categoryButtons), 1);

  connect(fCategoryListBox,SIGNAL(selected(int)),this,SLOT(categorySelected(int)));
  connect(fAddCategory,SIGNAL(clicked()),this,SLOT(addCategory()));
  connect(fModifyCategory,SIGNAL(clicked()),this,SLOT(modifyCategory()));
  connect(fRemoveCategory,SIGNAL(clicked()),this,SLOT(removeCategory()));

  QVBox * libraryPage = addVBoxPage(i18n("Library"),i18n("Local Music Library"),
				    UserIcon("local_lib", 16));

  QHBox * localBox = new QHBox(libraryPage);
  
  localBox->setSpacing(5);
  label = new QLabel(i18n("Local Library:"),localBox);
  fLocalLibraryEdit = new KLineEdit(localBox,"LocalLibraryEdit");
  label->setBuddy(fLocalLibraryEdit);
  fSelectLocalLibrary = new KPushButton(i18n("Adjust"),localBox,"AdjustClock");
  QIconSet iconSet = SmallIconSet(QString::fromLatin1("fileopen"));
  QPixmap pixMap = iconSet.pixmap(QIconSet::Small, QIconSet::Normal);
  fSelectLocalLibrary->setIconSet(iconSet);
  fSelectLocalLibrary->setFixedSize(pixMap.width()+8, pixMap.height()+8);
  QToolTip::add(fSelectLocalLibrary, i18n("Select Local Library"));  
  connect(fSelectLocalLibrary,SIGNAL(clicked()),
	  this,SLOT(selectLocalLibrary()));
  libraryPage->setStretchFactor(localBox,0);

  QVGroupBox * group = new QVGroupBox( i18n("Path and Filename Composition"), libraryPage);
  group->layout()->setSpacing( KDialog::spacingHint() );
  
  QHBox * savePathBox = new QHBox(group);
  
  label = new QLabel(i18n("Path:"),savePathBox);
  fSavePathEdit = new KLineEdit(group,"SavePathEdit");
  label->setBuddy(fSavePathEdit);

  QHBox * saveNameBox = new QHBox(group);
  
  label = new QLabel(i18n("Filename:"),saveNameBox);
  fSaveNameEdit = new KLineEdit(group,"SaveNameEdit");
  label->setBuddy(fSaveNameEdit);
  
  label = new QLabel(i18n("<b>%artist</b> will be replaced with the name of the Artist."),group);
  label = new QLabel(i18n("<b>%album</b> will be replaced with the name of the Album."),group);
  label = new QLabel(i18n("<b>%title</b> will be replaced with the Title of the track."),group);
  label = new QLabel(i18n("<b>%track</b> will be replaced with the tracknumber."),group);

  libraryPage->setStretchFactor(group,0);
  libraryPage->setStretchFactor(new QWidget(libraryPage),1);

  fTimer = new QTimer(this);
  connect(fTimer, SIGNAL(timeout()), this, SLOT(timeout()));
  fTimer->start(1000);
  
  fOwnerStringModified = false;
  fClockModified = false;

  fBusy = false;
}

KZenConfigDialog::~KZenConfigDialog()
{
  
}

void KZenConfigDialog::setup()
{
  fBusy = true;
  fOwnerEdit->setText(fOwnerString);
  fSmartImport->setChecked(KZenConfig::get()->smartImport());
  fLocalLibraryEdit->setText(KZenConfig::get()->localLibrary().path());
  fSavePathEdit->setText(KZenConfig::get()->savePath());
  fSaveNameEdit->setText(KZenConfig::get()->saveName());
  fBusy = false;
}

void KZenConfigDialog::acceptData()
{
  fOwnerString = fOwnerEdit->text().stripWhiteSpace();

  KZenConfig::get()->setSmartImport(fSmartImport->isChecked());

  KURL url;
  url.setPath(fLocalLibraryEdit->text().stripWhiteSpace());
  if (url.isValid() && url.isLocalFile()) {
    kdDebug() << "Library=" << url.path() << endl;
    KZenConfig::get()->setLocalLibrary(url);
  }
  
  KZenConfig::get()->setSavePath(fSavePathEdit->text().stripWhiteSpace());
  kdDebug() << "SavePath=" << fSavePathEdit->text().stripWhiteSpace() << endl;

  KZenConfig::get()->setSaveName(fSaveNameEdit->text().stripWhiteSpace());
  kdDebug() << "SaveName=" << fSaveNameEdit->text().stripWhiteSpace() << endl;
}

void KZenConfigDialog::setOwnerString(const QString & owner)
{
  fOwnerString = owner;
}

void KZenConfigDialog::setClock(const QDateTime & datetime)
{
  fBusy = true;
  fClockWidget->setDateTime(datetime);
  fBusy = false;
}

const QDateTime & KZenConfigDialog::KZenConfigDialog::getClock()
{
  fClock = fClockWidget->dateTime();
  return fClock;
}

void KZenConfigDialog::ownerChanged(const QString & o)
{
  if (!fBusy) {
    kdDebug() << "ownerChanged " << o << endl;
    fOwnerStringModified  = true;
  }
}

void KZenConfigDialog::clockChanged(const QDateTime & dt)
{
  if (!fBusy) {
    kdDebug() << "clockChanged " << dt << endl;
    fClockModified = true;
  }
}

void KZenConfigDialog::clockAdjust()
{
  fClock = QDateTime::currentDateTime();
  fClockWidget->setDateTime(fClock);
}

void KZenConfigDialog::timeout()
{
  fClock = fClockWidget->dateTime();
  setClock(fClock.addSecs(1));
}

void KZenConfigDialog::languageChange()
{
  setCaption(i18n("Configuration - KZenExplorer"));
}

void KZenConfigDialog::categorySelected(int idx)
{
  fCategoryEdit->setText(fCategoryListBox->text(idx));
}

void KZenConfigDialog::addCategory()
{
  QString text = fCategoryEdit->text();
  if (text.isNull() || text.isEmpty()) return;
  text = text.stripWhiteSpace();

  fCategoryListBox->insertItem(text);
  gCategories->addCategory(text);
}

void KZenConfigDialog::modifyCategory()
{
  QString text = fCategoryEdit->text();
  if (text.isNull() || text.isEmpty()) return;
  text = text.stripWhiteSpace();
  
  int idx = fCategoryListBox->currentItem();
  if (text!=fCategoryListBox->text(idx)) {
    fCategoryListBox->changeItem(text,idx);
    (gCategories->categories())[idx] = text;

    fExplorer->getView()->refresh();
  }
}

void KZenConfigDialog::removeCategory()
{
  int idx = fCategoryListBox->currentItem();
  fCategoryListBox->removeItem(idx);
  (gCategories->categories()).remove((gCategories->categories())[idx]);

  KZenTrack * track;
  for (uint i=0;i<fExplorer->getTracks()->count();i++) {
    track = fExplorer->getTracks()->at(i);
    track->removeCategory(idx);
  }
  
  fExplorer->getView()->refresh();
}

void KZenConfigDialog::selectLocalLibrary()
{
  KURL url = KDirSelectDialog::selectDirectory(KZenConfig::get()->localLibrary().url(),
					       true);
  if (url.isValid() && url.isLocalFile()) {
    fLocalLibraryEdit->setText(url.path());
  }
}

#include "kzenconfigdialog.moc"
